@import url("https://fonts.googleapis.com/css?family=Open+Sans:300,400,400i,600,700");

[data-init="HtmlBlock"] * {
  line-height: 1.4em;
}

[data-init="HtmlBlock"] h1 {
  color: var(--body-color, #313131);
  font: normal 2em/1.4em var(--font-family-sans-serif);
  letter-spacing: 1px;
  margin: 0 0 1.416em;
}

[data-init="HtmlBlock"] h2 {
  color: #646464;
  font: normal 1.2em/1.2em var(--font-family-sans-serif);
  letter-spacing: 1px;
  margin-bottom: calc((var(--baseline, 20px) * 0.75));
  -webkit-font-smoothing: antialiased;
}

[data-init="HtmlBlock"] h3,
[data-init="HtmlBlock"] h4,
[data-init="HtmlBlock"] h5,
[data-init="HtmlBlock"] h6 {
  margin: 0 0 calc((var(--baseline, 20px) / 2));
  font-weight: 600;
}

[data-init="HtmlBlock"] h3 {
  font-size: 1.2em;
}

[data-init="HtmlBlock"] h4 {
  font-size: 1em;
}

[data-init="HtmlBlock"] h5 {
  font-size: 0.83em;
}

[data-init="HtmlBlock"] h6 {
  font-size: 0.75em;
}

[data-init="HtmlBlock"] p {
  margin-bottom: 1.416em;
  font-size: 1em;
  line-height: 1.6em !important;
  color: var(--body-color, #313131);
}

[data-init="HtmlBlock"] em,
[data-init="HtmlBlock"] i {
  font-style: italic;
}

[data-init="HtmlBlock"] em span,
[data-init="HtmlBlock"] i span {
  font-style: italic;
}

[data-init="HtmlBlock"] strong,
[data-init="HtmlBlock"] b {
  font-weight: bold;
}

[data-init="HtmlBlock"] strong span,
[data-init="HtmlBlock"] b span {
  font-weight: bold;
}

[data-init="HtmlBlock"] p+p,
[data-init="HtmlBlock"] ul+p,
[data-init="HtmlBlock"] ol+p {
  margin-top: var(--baseline, 20px);
}

[data-init="HtmlBlock"] blockquote {
  margin: 1em calc((var(--baseline, 20px) * 2));
}

[data-init="HtmlBlock"] ol,
[data-init="HtmlBlock"] ul {
  padding: 0 0 0 1em;
  margin: 1em 0;
  color: var(--body-color, #313131);
}

[data-init="HtmlBlock"] ol li,
[data-init="HtmlBlock"] ul li {
  margin-bottom: 0.708em;
}

[data-init="HtmlBlock"] ol {
  list-style: decimal outside none;
}

[data-init="HtmlBlock"] ul {
  list-style: disc outside none;
}

[data-init="HtmlBlock"] a:link,
[data-init="HtmlBlock"] a:visited,
[data-init="HtmlBlock"] a:hover,
[data-init="HtmlBlock"] a:active,
[data-init="HtmlBlock"] a:focus {
  color: var(--blue, #0075b4);
}

[data-init="HtmlBlock"] img {
  max-width: 100%;
  height: auto;
}

[data-init="HtmlBlock"] pre {
  margin: 1em 0;
  color: var(--body-color, #313131);
  font-family: monospace, serif;
  font-size: 1em;
  white-space: pre-wrap;
  word-wrap: break-word;
}

[data-init="HtmlBlock"] code {
  color: var(--body-color, #313131);
  font-family: monospace, serif;
  background: none;
  padding: 0;
}

[data-init="HtmlBlock"] table {
  width: 100%;
  margin: var(--baseline, 20px) 0;
  border-collapse: collapse;
  font-size: 16px;
}

[data-init="HtmlBlock"] table td,
[data-init="HtmlBlock"] table th {
  margin: var(--baseline, 20px) 0;
  padding: calc((var(--baseline, 20px) / 2));
  border: 1px solid var(--gray-l3, #c8c8c8);
  font-size: 14px;
}

[data-init="HtmlBlock"] table td.cont-justified-left,
[data-init="HtmlBlock"] table th.cont-justified-left {
  text-align: left !important;
}

[data-init="HtmlBlock"] table td.cont-justified-right,
[data-init="HtmlBlock"] table th.cont-justified-right {
  text-align: right !important;
}

[data-init="HtmlBlock"] table td.cont-justified-center,
[data-init="HtmlBlock"] table th.cont-justified-center {
  text-align: center !important;
}

[data-init="HtmlBlock"] th {
  background: #eee;
  font-weight: bold;
}

[data-init="HtmlBlock"] .wrapper-modal-image .modal-ui-icon {
  position: absolute;
  display: block;
  padding: calc((var(--baseline, 20px) / 4)) 7px;
  border-radius: 5px;
  opacity: 0.9;
  background: var(--white, #fff);
  color: var(--black, #000);
  border: 2px solid var(--black, #000);
}

[data-init="HtmlBlock"] .wrapper-modal-image .modal-ui-icon .label {
  font-weight: bold;
}

[data-init="HtmlBlock"] .wrapper-modal-image .modal-ui-icon i {
  font-style: normal;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-link {
  position: relative;
  display: block;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-link .action-fullscreen {
  display: none;
  top: 10px;
  left: 10px;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-link:hover .action-fullscreen {
  display: block;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal {
  position: fixed;
  top: 0;
  left: 0;
  display: none;
  height: 100%;
  width: 100%;
  background-color: rgba(0, 0, 0, 0.7);
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content {
  position: relative;
  top: 2.5%;
  display: block;
  height: 95%;
  width: 95%;
  margin: auto;
  overflow: hidden;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-wrapper {
  position: relative;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-wrapper img {
  position: relative;
  display: block;
  max-width: 100%;
  max-height: 100%;
  margin: auto;
  cursor: default;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .action-close {
  top: 10px;
  right: 10px;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls {
  position: absolute;
  right: 10px;
  bottom: 10px;
  margin: 0;
  padding: 0;
  list-style: none;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls .image-control {
  position: relative;
  display: inline-block;
  margin: 0;
  padding: 0;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls .image-control .modal-ui-icon {
  position: relative;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls .image-control .modal-ui-icon.action-zoom-in {
  margin-right: calc((var(--baseline, 20px) / 4));
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls .image-control .modal-ui-icon.action-zoom-out {
  margin-left: calc((var(--baseline, 20px) / 4));
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal .image-content .image-controls .image-control .modal-ui-icon.is-disabled {
  opacity: 0.5;
  cursor: default;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal.image-is-fit-to-screen {
  display: block;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal.image-is-fit-to-screen .image-content .image-wrapper {
  top: 0 !important;
  left: 0 !important;
  width: 100% !important;
  height: 100% !important;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal.image-is-fit-to-screen .image-content .image-wrapper img {
  top: 0 !important;
  left: 0 !important;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal.image-is-zoomed {
  display: block;
}

[data-init="HtmlBlock"] .wrapper-modal-image .image-modal.image-is-zoomed .image-content .image-wrapper img {
  max-width: none;
  max-height: none;
  margin: 0;
  cursor: move;
}
