/* CSS for WordCloudBlock */

@import url("https://fonts.googleapis.com/css?family=Open+Sans:300,400,400i,600,700");

.input-cloud {
    margin: calc((var(--baseline, 20px) / 4));
}

.result_cloud_section {
    display: none;
    width: 0;
    height: 0;
}

.result_cloud_section.active {
    display: block;
    width: 100%;
    height: auto;
    margin-top: 1em;
}

.result_cloud_section.active h3 {
    font-size: 100%;
}

.your_words {
    font-size: 0.85em;
    display: block;
}
