@import url("https://fonts.googleapis.com/css?family=Open+Sans:300,400,400i,600,700");

.inline {
    display: inline;
}

  h3 {
    margin-top: 0;
    margin-bottom: calc((var(--baseline, 20px) * 0.75));
    color: #fe57a1;
    font-size: 1.9em;
}

  h3.problem-header .staff {
    margin-top: calc((var(--baseline, 20px) * 1.5));
    font-size: 80%;
}

 p {
    text-align: justify;
    font-weight: bold;
}

  .poll_answer {
    margin-bottom: var(--baseline, 20px);
}

  .poll_answer.short {
    clear: both;
}

  .poll_answer .question {
    height: auto;
    clear: both;
    min-height: 30px;
}

  .poll_answer .question.short {
    clear: none;
    width: 30%;
    display: inline;
    float: left;
}

  .poll_answer.short .question.short .button.short {
    -webkit-appearance: none;
    -webkit-background-clip: padding-box;
    -webkit-border-image: none;
    -webkit-box-align: center;
    -webkit-box-shadow: white 0px 1px 0px 0px inset;
    -webkit-font-smoothing: antialiased;
    -webkit-rtl-ordering: logical;
    -webkit-user-select: text;
    -webkit-writing-mode: horizontal-tb;
    background-clip: padding-box;
    background-color: #eeeeee;
    background-image: -webkit-linear-gradient(top, #eeeeee, #d2d2d2);
    border-bottom-color: #cacaca;
    border-bottom-left-radius: 3px;
    border-bottom-right-radius: 3px;
    border-bottom-style: solid;
    border-bottom-width: 1px;
    border-left-color: #cacaca;
    border-left-style: solid;
    border-left-width: 1px;
    border-right-color: #cacaca;
    border-right-style: solid;
    border-right-width: 1px;
    border-top-color: #cacaca;
    border-top-left-radius: 3px;
    border-top-right-radius: 3px;
    border-top-style: solid;
    border-top-width: 1px;
    box-shadow: white 0px 1px 0px 0px inset;
    box-sizing: border-box;
    color: #333333;
    /* display: inline-block; */
    display: inline;
    float: left;
    font-family: 'Open Sans', Verdana, Geneva, sans-serif;
    font-size: 13px;
    font-style: normal;
    font-variant: normal;
    font-weight: bold;
    letter-spacing: normal;
    line-height: 25.59375px;
    margin-bottom: calc((var(--baseline, 20px) * 0.75));
    margin: 0;
    padding: 0px;
    text-align: center;
    text-decoration: none;
    text-indent: 0px;
    text-shadow: #f8f8f8 0px 1px 0px;
    text-transform: none;
    vertical-align: top;
    white-space: pre-line;
    width: 25px;
    height: 25px;
    word-spacing: 0px;
    writing-mode: lr-tb;
}

 .poll_answer.short .question.short .button.short.answered {
    -webkit-box-shadow: #61b8e1 0px 1px 0px 0px inset;
    background-color: #1d9dd9;
    background-image: -webkit-linear-gradient(top, #1d9dd9, #0e7cb0);
    border-bottom-color: #0d72a2;
    border-left-color: #0d72a2;
    border-right-color: #0d72a2;
    border-top-color: #0d72a2;
    box-shadow: #61b8e1 0px 1px 0px 0px inset;
    color: white;
    text-shadow: #076794 0px 1px 0px;
    background-image: none;
}

 .poll_answer .question .text {
    display: inline;
    float: left;
    width: 80%;
    text-align: left;
    min-height: 30px;
    margin-left: var(--baseline, 20px);
    height: auto;
    margin-bottom: var(--baseline, 20px);
}

 .poll_answer.short .question.short .text.short {
    width: 100px;
}

 .poll_answer .stats {
    min-height: 40px;
    margin-top: var(--baseline, 20px);
    clear: both;
}

 .poll_answer.short .stats.short {
    margin-top: 0;
    clear: none;
    display: inline;
    float: right;
    width: 70%;
}

 .poll_answer .stats .bar {
    width: 75%;
    height: 20px;
    border: 1px solid black;
    display: inline;
    float: left;
    margin-right: calc((var(--baseline, 20px) / 2));
}

 .poll_answer.short .stats.short .bar.short {
    width: 65%;
    height: 20px;
    margin-top: 3px;
}

 .poll_answer .stats .bar .percent {
    background-color: gray;
    width: 0;
    height: 20px;
}

 .poll_answer .stats .number {
    width: 80px;
    display: inline;
    float: right;
    height: 28px;
    text-align: right;
}

 .poll_answer.short .stats.short .number.short {
    width: 120px;
    height: auto;
}

 .poll_answer.answered {
    -webkit-box-shadow: #61b8e1 0 1px 0 0 inset;
    background-color: #1d9dd9;
    background-image: -webkit-linear-gradient(top, #1d9dd9, #0e7cb0);
    border-bottom-color: #0d72a2;
    border-left-color: #0d72a2;
    border-right-color: #0d72a2;
    border-top-color: #0d72a2;
    box-shadow: #61b8e1 0 1px 0 0 inset;
    color: white;
    text-shadow: #076794 0 1px 0;
}

 .button.reset-button {
    clear: both;
    float: right;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .question.short {
    clear: none;
    width: 30%;
    display: inline;
    float: left;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .question .button {
    -webkit-appearance: none;
    -webkit-background-clip: padding-box;
    -webkit-border-image: none;
    -webkit-box-align: center;
    -webkit-box-shadow: white 0px 1px 0px 0px inset;
    -webkit-font-smoothing: antialiased;
    -webkit-rtl-ordering: logical;
    -webkit-user-select: text;
    -webkit-writing-mode: horizontal-tb;
    background-clip: padding-box;
    background-color: #eeeeee;
    background-image: -webkit-linear-gradient(top, #eeeeee, #d2d2d2);
    border-bottom-color: #cacaca;
    border-bottom-left-radius: 3px;
    border-bottom-right-radius: 3px;
    border-bottom-style: solid;
    border-bottom-width: 1px;
    border-left-color: #cacaca;
    border-left-style: solid;
    border-left-width: 1px;
    border-right-color: #cacaca;
    border-right-style: solid;
    border-right-width: 1px;
    border-top-color: #cacaca;
    border-top-left-radius: 3px;
    border-top-right-radius: 3px;
    border-top-style: solid;
    border-top-width: 1px;
    box-shadow: white 0px 1px 0px 0px inset;
    box-sizing: border-box;
    color: #333333;
    /* display: inline-block; */
    display: inline;
    float: left;
    font-family: 'Open Sans', Verdana, Geneva, sans-serif;
    font-size: 13px;
    font-style: normal;
    font-variant: normal;
    font-weight: bold;
    letter-spacing: normal;
    line-height: 25.59375px;
    margin-bottom: calc((var(--baseline) * 0.75));
    margin: 0;
    padding: 0px;
    text-align: center;
    text-decoration: none;
    text-indent: 0px;
    text-shadow: #f8f8f8 0px 1px 0px;
    text-transform: none;
    vertical-align: top;
    white-space: pre-line;
    width: 25px;
    height: 25px;
    word-spacing: 0px;
    writing-mode: lr-tb;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .question .button.answered {
    -webkit-box-shadow: #61b8e1 0px 1px 0px 0px inset;
    background-color: #1d9dd9;
    background-image: -webkit-linear-gradient(top, #1d9dd9, #0e7cb0);
    border-bottom-color: #0d72a2;
    border-left-color: #0d72a2;
    border-right-color: #0d72a2;
    border-top-color: #0d72a2;
    box-shadow: #61b8e1 0px 1px 0px 0px inset;
    color: white;
    text-shadow: #076794 0px 1px 0px;
    background-image: none;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .question .text {
    display: inline;
    float: left;
    width: 80%;
    text-align: left;
    min-height: 30px;
    margin-left: var(--baseline);
    height: auto;
    margin-bottom: var(--baseline);
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .question .text.short {
    width: 100px;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats {
    min-height: 40px;
    margin-top: var(--baseline);
    clear: both;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats.short {
    margin-top: 0;
    clear: none;
    display: inline;
    float: right;
    width: 70%;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats .bar {
    width: 75%;
    height: 20px;
    border: 1px solid black;
    display: inline;
    float: left;
    margin-right: calc((var(--baseline) / 2));
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats .bar.short {
    width: 65%;
    height: 20px;
    margin-top: 3px;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats .bar .percent {
    background-color: gray;
    width: 0;
    height: 20px;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats .number {
    width: 80px;
    display: inline;
    float: right;
    height: 28px;
    text-align: right;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer .stats .number.short {
    width: 120px;
    height: auto;
}

.xmodule_display.xmodule_PollBlock div.poll_question .poll_answer.answered {
    -webkit-box-shadow: #61b8e1 0 1px 0 0 inset;
    background-color: #1d9dd9;
    background-image: -webkit-linear-gradient(top, #1d9dd9, #0e7cb0);
    border-bottom-color: #0d72a2;
    border-left-color: #0d72a2;
    border-right-color: #0d72a2;
    border-top-color: #0d72a2;
    box-shadow: #61b8e1 0 1px 0 0 inset;
    color: white;
    text-shadow: #076794 0 1px 0;
}

.xmodule_display.xmodule_PollBlock div.poll_question .button.reset-button {
    clear: both;
    float: right;
}