"""Init for the xblocks_contrib package."""

from .annotatable import AnnotatableBlock
from .discussion import DiscussionXBlock
from .html import HtmlBlock
from .lti import LTIBlock
from .poll import PollBlock
from .problem import ProblemBlock
from .video import VideoBlock
from .word_cloud import WordCloudBlock

__version__ = "0.8.1"
