<div
    id="{{ element_id }}"
    class="{{ element_class }}"
>
    <!-- Hidden field to read configuration JSON from. -->
    <div class="{{ element_class }}_div" id="{{ element_id }}_json" style="display: none;">{{ configuration_json|safe}}</div>
</div>
